--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Sunrise Sunset v1.02
--		====================
--
--
--
--	by Taraezor / Chris Birch
--
--	29th March 2017
--
--
--------------------------------------------------------------------------------------------------------------------------------------------

local addonName, L = ...

local random = random
local gmatch = string.gmatch
local lower = string.lower
local sub = string.sub

local CalendarGetDate = CalendarGetDate
local CalendarGetDayEvent = CalendarGetDayEvent
local CalendarSetAbsMonth = CalendarSetAbsMonth
local CalendarSetMonth = CalendarSetMonth
local GameTime_GetFormattedTime = GameTime_GetFormattedTime
local GameTime_GetLocalTime = GameTime_GetLocalTime
local GetAddOnMetadata = GetAddOnMetadata
local PlaySoundFile = PlaySoundFile
local PlaySoundKitID = PlaySoundKitID
local StopSound = StopSound
local UnitFactionGroup = UnitFactionGroup

--------------------------------------------------------------------------------------------------------------------------------------------
-- Print Colours - Sunrise/Sunset Theme
--------------------------------------------------------------------------------------------------------------------------------------------

local pc_colour_Prefix		= "\124cFFFF4500"
local pc_colour_Highlight	= "\124cFFFFA500"
local pc_colour_PlainText	= "\124cFFFF8C00"


local function printPC( message )
	DEFAULT_CHAT_FRAME:AddMessage( pc_colour_Prefix.. addonName.. ": ".. pc_colour_PlainText.. message.. "\124r" )
end

--------------------------------------------------------------------------------------------------------------------------------------------
-- Estimation
--------------------------------------------------------------------------------------------------------------------------------------------

function SunriseSunset_EstimateTime()

	local _, monthCur, dayCur, yearCur = CalendarGetDate()

	CalendarSetAbsMonth( monthCur, yearCur )

	local lookdownDay, foundHour, foundMinute, found = dayCur+1, 0, 0, false
	local lookdownMonth = monthCur
	local daysInMonth = ( ( monthCur == 2 ) and ( yearCur % 4 == 0 ) and ( ( yearCur % 100 ~= 0 ) or ( yearCur % 400 == 0 ) ) )
					and 29 or ( "\31\28\31\30\31\30\31\31\30\31\30\31" ):byte( monthCur )

	for i = 0, 30 do
		lookdownDay = lookdownDay - 1

		if lookdownDay < 1 then
			lookdownMonth = ( monthCur - 1 ) == 0 and 12 or ( monthCur - 1 )
			local lookdownYear = ( lookdownMonth == 12 ) and ( yearCur - 1 ) or yearCur
			CalendarSetMonth( -1 )
			lookdownDay = ( ( lookdownMonth == 2 ) and ( lookdownYear % 4 == 0 ) and ( ( lookdownYear % 100 ~= 0 ) or ( lookdownYear % 400 == 0 ) ) )
						and 29 or ( "\31\28\31\30\31\30\31\31\30\31\30\31" ):byte( lookdownMonth )
		end

		for j = 1, 100 do
			local title, hour, minute, calendarType, sequenceType, eventType, texture = CalendarGetDayEvent( 0, lookdownDay, j )
			if not title then
				break
			end
			if ( calendarType == "HOLIDAY" ) then
				if ( title == CALENDAR_FILTER_DARKMOON ) then
					if ( sequenceType == "START" ) or ( sequenceType == "END" ) then
						found = true
						foundHour, foundMinute = hour, minute
						break
					end
				elseif ( texture == "Calendar_Brewfest" ) or ( texture == "Calendar_ChildrensWeek" )  or 
					( texture == "Calendar_DayOfTheDead" ) or ( texture == "Calendar_LoveInTheAir" ) or 
					( texture == "Calendar_LunarFestival" ) or ( texture == "Calendar_Midsummer" ) or
					( texture == "Calendar_Noblegarden" ) or ( texture == "Calendar_PiratesDay" ) then
					if ( sequenceType == "START" ) or ( sequenceType == "END" ) then
						found = true
						foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
						break
					end
				elseif ( texture == "Calendar_WinterVeil" ) then
					if ( sequenceType == "START" ) then
						found = true
						foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
						break
					elseif ( sequenceType == "END" ) then
						found = true
						foundHour, foundMinute = ( 24 - ( 6 - hour ) ), minute
						break
					end
				elseif ( texture == "Calendar_HallowsEnd" ) then
					if ( sequenceType == "START" ) then
						found = true
						foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
						break
					elseif ( sequenceType == "END" ) then
						found = true
						foundHour, foundMinute = ( 24 - ( 11 - hour ) ), minute
						break
					end
				elseif ( texture == "Calendar_HarvestFestival" ) then
					if ( lookdownMonth >= 11 ) then			-- Pilgrim's
						if ( sequenceType == "START" ) then
							found = true
							foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
							break
						elseif ( sequenceType == "END" ) then
							found = true
							foundHour, foundMinute = ( 24 - ( 9 - hour ) ), minute
							break
						end
					else						-- Harvest
						if ( sequenceType == "START" ) then
							found = true
							foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
							break
						elseif ( sequenceType == "END" ) then
							found = true
							foundHour, foundMinute = ( 24 - ( 10 - hour ) ), minute
							break
						end
					end
				end
			end
		end
		if ( found == true ) then break end
	end

	CalendarSetAbsMonth( monthCur, yearCur )

	return foundHour, foundMinute
end

--------------------------------------------------------------------------------------------------------------------------------------------
-- Minimap Tooltip Hook
--------------------------------------------------------------------------------------------------------------------------------------------

function SunriseSunset_SetTooltip()

	local offsetH, offsetM = SunriseSunset_EstimateTime()

	-- Sunrise is 5:30AM and sunset is 9:00 PM

	local dawnM = 30 + offsetM
	local dawnH = 5 + offsetH
	if dawnM >= 60 then
		dawnM = dawnM - 60
		dawnH = dawnH + 1
	end
	dawnH = ( dawnH >= 24 ) and ( dawnH - 24 ) or dawnH

	local duskM = offsetM
	local duskH = 21 + offsetH
	duskH = ( duskH >= 24 ) and ( duskH - 24 ) or duskH

	local _, hour, minute = GameTime_GetLocalTime( true )

	local worldM = minute - offsetM
	local worldH = hour - offsetH
	if worldM < 0 then
		worldM = worldM + 60
		worldH = worldH - 1
	end
	worldH = ( worldH < 0 ) and ( worldH + 24 ) or worldH

	if ( SunriseSunsetDB.sunrise == "y" ) or ( SunriseSunsetDB.sunset == "y" ) or ( SunriseSunsetDB.world == "y" ) then
		GameTooltip:AddLine( " ", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
	end

	if ( SunriseSunsetDB.sunrise == "y" ) then
		GameTooltip:AddDoubleLine( L["Sunrise"], GameTime_GetFormattedTime( dawnH, dawnM, true ),
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end

	if ( SunriseSunsetDB.sunset == "y" ) then
		GameTooltip:AddDoubleLine( L["Sunset"], GameTime_GetFormattedTime( duskH, duskM, true ),
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end
	
	if ( SunriseSunsetDB.world == "y" ) then
		GameTooltip:AddDoubleLine( "((".. L["Central Server Time"].. "))", GameTime_GetFormattedTime( worldH, worldM, true ),
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end
	
	GameTooltip:Show()
end

--------------------------------------------------------------------------------------------------------------------------------------------
-- Slash Commands
--------------------------------------------------------------------------------------------------------------------------------------------

local soundkitHandle = nil

SLASH_SunriseSunset1, SLASH_SunriseSunset2, SLASH_SunriseSunset3 = "/sunrise", "/sunset", "/ss"

SlashCmdList[ "SunriseSunset" ] = function( options )

	local options1

	for v in gmatch( options, "%S+" ) do
		v = lower( v )
		options1 = sub( v,1,1 )
		break
	end

	if ( options1 == "?" ) or ( options1 == nil ) then
		printPC( pc_colour_Highlight.. L["Macro Commands"].. ":\n"..
			pc_colour_Highlight.. "r".. pc_colour_PlainText.. " = ".. L["Sunrise"].. "? (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "s".. pc_colour_PlainText.. " = ".. L["Sunset"].. "? (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "t".. pc_colour_PlainText.. " = ".. L["Central Server Time"].. "? (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "v".. pc_colour_PlainText.. " = ".. L["Show the version number"].. "\n")

	elseif ( options1 == "r" ) then
		SunriseSunsetDB.sunrise = ( SunriseSunsetDB.sunrise == "y" ) and "n" or "y"
		if ( SunriseSunsetDB.sunrise == "y" ) then
			printPC( L["Sunrise"].. ": ".. pc_colour_Highlight.. L["On"] )
		else
			printPC( L["Sunrise"].. ": ".. pc_colour_Highlight.. L["Off"] )
		end

	elseif ( options1 == "s" ) then
		SunriseSunsetDB.sunset = ( SunriseSunsetDB.sunset == "y" ) and "n" or "y"
		if ( SunriseSunsetDB.sunset == "y" ) then
			printPC( L["Sunset"].. ": ".. pc_colour_Highlight.. L["On"] )
		else
			printPC( L["Sunset"].. ": ".. pc_colour_Highlight.. L["Off"] )
		end

	elseif ( options1 == "t" ) then
		SunriseSunsetDB.world = ( SunriseSunsetDB.world == "y" ) and "n" or "y"
		if ( SunriseSunsetDB.world == "y" ) then
			printPC( L["Central Server Time"].. ": ".. pc_colour_Highlight.. L["On"] )
		else
			printPC( L["Central Server Time"].. ": ".. pc_colour_Highlight.. L["Off"] )
		end

	elseif ( options1 == "v" ) then
		local version = GetAddOnMetadata( addonName, "Version" )
		printPC( L["Version: "].. pc_colour_Highlight.. version )

	-- Easter Eggs. Enjoy!

	elseif ( options1 == "z" ) then
		if ( soundkitHandle ) then
			StopSound( soundkitHandle, 5000 )
			soundkitHandle = nil
		else
			local soundSelection = random( 2 )
			if ( soundSelection == 1 ) then
				_, soundkitHandle = PlaySoundKitID( 10896 )		-- Lament of the Highborne
			else
				local faction = UnitFactionGroup( "player" )		-- Winter Veil
				if ( faction == "Alliance" ) then
					_, soundkitHandle = PlaySoundKitID( 79294 )
				elseif ( faction == "Horde" ) then
					_, soundkitHandle = PlaySoundKitID( 79122 )
				else
					_, soundkitHandle = PlaySoundKitID( 79123 )
				end
			end
		end
	end
end